@@ -9,6 +9,9 @@ use std::ffi::OsStr;
 use log::debug;
 use std::io::{stdin, stdout, Write};
 use std::path::{Path, PathBuf};
+use std::sync::{Arc, Mutex};
+use std::thread;
+use std::time::Duration;
 use std::time::Instant;
 use termion::cursor::DetectCursorPos;
 use termion::event::Key;
@@ -16,6 +19,8 @@ use termion::input::TermRead;
 use termion::raw::IntoRawMode;
 use termion::{clear, cursor, screen};
 
+const DETECTION_INTERVAL: u64 = 500;
+
 pub fn run(arg: PathBuf) -> Result<(), MyError> {
     debug!("Initial setup starts.");
 
@@ -30,38 +35,78 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
     make_session(&session_file_path).unwrap_or_else(|_| panic!("Cannot make session file."));
 
     if !&arg.exists() {
-        println!("Invalid path: {}\n`fx -h` shows help.", &arg.display());
+        println!(
+            "Invalid path or argument: {}\n`fx -h` shows help.",
+            &arg.display()
+        );
         return Ok(());
     }
 
+    //Initialize app state
     let mut state = State::new();
     state.trash_dir = trash_dir_path;
     state.current_dir = arg.canonicalize()?;
 
     let mut filtered = false;
 
-    let mut nums = Num::new();
+    //Initialize num as Arc
+    let nums = Num::new();
+    let nums_run = Arc::new(Mutex::new(nums));
+    let nums_detect = nums_run.clone();
 
-    let mut screen = screen::AlternateScreen::from(stdout().into_raw_mode().unwrap());
+    //Initialize screen as Arc
+    let screen = screen::AlternateScreen::from(stdout().into_raw_mode().unwrap());
+    let screen_run = Arc::new(Mutex::new(screen));
+    let screen_detect = screen_run.clone();
 
+    //Update list, print and flush
     print!("{}", cursor::Hide);
-
     state.update_list()?;
     clear_and_show(&state.current_dir);
     state.list_up(nums.skip);
-
     state.move_cursor(&nums, STARTING_POINT);
-    screen.flush()?;
+    let mut init_screen = screen_run.lock().unwrap();
+    init_screen.flush()?;
+    drop(init_screen);
 
+    //Initialize cursor move memo
     let mut p_memo_v: Vec<ParentMemo> = Vec::new();
     let mut c_memo_v: Vec<ChildMemo> = Vec::new();
+
+    //Prepare state as Arc
+    let state_run = Arc::new(Mutex::new(state));
+    let state_detect = state_run.clone();
+
+    //Detect terminal window change
+    let interval = Duration::from_millis(DETECTION_INTERVAL);
+    thread::spawn(move || loop {
+        thread::sleep(interval);
+        let (column, row) = termion::terminal_size().unwrap();
+        let mut state = state_detect.lock().unwrap();
+        let mut nums = nums_detect.lock().unwrap();
+        if column != state.layout.terminal_column || row != state.layout.terminal_row {
+            if state.layout.y < row {
+                let cursor_pos = state.layout.y;
+                state.refresh(column, row, &nums, cursor_pos);
+            } else {
+                let diff = state.layout.y + 1 - row;
+                nums.index -= diff as usize;
+                state.refresh(column, row, &nums, row - 1);
+            }
+            let mut screen = screen_detect.lock().unwrap();
+            screen.flush().unwrap();
+        }
+    });
+
     let mut stdin = stdin().keys();
 
     'main: loop {
-        let len = state.list.len();
-        let (_, y) = screen.cursor_pos()?;
         let input = stdin.next();
-
+        let mut state = state_run.lock().unwrap();
+        let mut screen = screen_run.lock().unwrap();
+        let mut nums = nums_run.lock().unwrap();
+        let len = state.list.len();
+        let y = state.layout.y;
         if let Some(Ok(key)) = input {
             match key {
                 //Go up. If lists exceed max-row, lists "scrolls" before the top of the list
@@ -85,14 +130,14 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
 
                 //Go down. If lists exceed max-row, lists "scrolls" before the bottom of the list
                 Key::Char('k') | Key::Up => {
-                    if y == STARTING_POINT {
+                    if nums.index == 0 {
                         continue;
-                    } else if y == STARTING_POINT + 3 && nums.skip != 0 {
+                    } else if y <= STARTING_POINT + 3 && nums.skip != 0 {
                         nums.go_up();
                         nums.dec_skip();
                         clear_and_show(&state.current_dir);
                         state.list_up(nums.skip);
-                        state.move_cursor(&nums, STARTING_POINT + 3);
+                        state.move_cursor(&nums, y);
                     } else {
                         nums.go_up();
                         print!(" ");
@@ -147,7 +192,8 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                         nums.go_bottom(len - 1);
                         clear_and_show(&state.current_dir);
                         state.list_up(nums.skip);
-                        state.move_cursor(&nums, state.layout.terminal_row - 1);
+                        let cursor_pos = state.layout.terminal_row - 1;
+                        state.move_cursor(&nums, cursor_pos);
                     } else {
                         nums.reset();
                         nums.go_bottom(len - 1);
@@ -169,6 +215,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                 }
                                 print!("{}", screen::ToAlternateScreen);
                                 clear_and_show(&state.current_dir);
+                                state.update_list()?;
                                 state.list_up(nums.skip);
                                 print!("{}", cursor::Hide);
                                 state.move_cursor(&nums, y);
@@ -183,7 +230,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         let cursor_memo = if !filtered {
                                             ParentMemo {
                                                 to_sym_dir: Some(state.current_dir.clone()),
-                                                num: nums.clone(),
+                                                num: *nums,
                                                 cursor_pos: y,
                                             }
                                         } else {
@@ -234,7 +281,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         let cursor_memo = if !filtered {
                                             ParentMemo {
                                                 to_sym_dir: None,
-                                                num: nums.clone(),
+                                                num: *nums,
                                                 cursor_pos: y,
                                             }
                                         } else {
@@ -259,7 +306,8 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         match c_memo_v.pop() {
                                             Some(memo) => {
                                                 if state.current_dir == memo.dir_path {
-                                                    nums = memo.num;
+                                                    nums.index = memo.num.index;
+                                                    nums.skip = memo.num.skip;
                                                     clear_and_show(&state.current_dir);
                                                     state.list_up(nums.skip);
                                                     state.move_cursor(&nums, memo.cursor_pos);
@@ -293,7 +341,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                             let cursor_memo = if !filtered {
                                 ChildMemo {
                                     dir_path: pre.clone(),
-                                    num: nums.clone(),
+                                    num: *nums,
                                     cursor_pos: y,
                                 }
                             } else {
@@ -321,7 +369,8 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         continue;
                                     }
                                     state.update_list()?;
-                                    nums = memo.num;
+                                    nums.index = memo.num.index;
+                                    nums.skip = memo.num.skip;
                                     clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
                                     state.move_cursor(&nums, memo.cursor_pos);
@@ -530,7 +579,8 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         state.select_to_bottom(start_pos);
                                         clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
-                                        state.move_cursor(&nums, state.layout.terminal_row - 1);
+                                        let cursor_pos = state.layout.terminal_row - 1;
+                                        state.move_cursor(&nums, cursor_pos);
                                     } else {
                                         nums.go_bottom(len - 1);
                                         state.select_to_bottom(start_pos);
@@ -1121,35 +1171,48 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     }
 
                                     if c == "z" && args.len() == 1 {
-                                        let output = std::process::Command::new("zoxide")
+                                        if let Ok(output) = std::process::Command::new("zoxide")
                                             .args(["query", args[0].trim()])
-                                            .output()?;
-                                        let output = output.stdout;
-                                        if !output.is_empty() {
-                                            let target_dir = std::str::from_utf8(&output);
-                                            match target_dir {
-                                                Err(e) => {
-                                                    print!("{}", cursor::Hide);
-                                                    print_warning(e, y);
-                                                    break 'command;
-                                                }
-                                                Ok(target_dir) => {
-                                                    let target_path =
-                                                        PathBuf::from(target_dir.trim());
-                                                    print_warning(target_path.to_str().unwrap(), y);
-                                                    state.current_dir =
-                                                        target_path.canonicalize()?;
-                                                    nums.reset();
-                                                    state.update_list()?;
-                                                    p_memo_v = Vec::new();
-                                                    c_memo_v = Vec::new();
-                                                    clear_and_show(&state.current_dir);
-                                                    state.list_up(nums.skip);
-                                                    print!("{}", cursor::Hide);
-                                                    state.move_cursor(&nums, STARTING_POINT);
-                                                    break 'command;
+                                            .output()
+                                        {
+                                            let output = output.stdout;
+                                            if output.is_empty() {
+                                                print_warning(
+                                                    "Keyword cannot match the database.",
+                                                    y,
+                                                );
+                                                break 'command;
+                                            } else {
+                                                let target_dir = std::str::from_utf8(&output);
+                                                match target_dir {
+                                                    Err(e) => {
+                                                        print!("{}", cursor::Hide);
+                                                        print_warning(e, y);
+                                                        break 'command;
+                                                    }
+                                                    Ok(target_dir) => {
+                                                        let target_path =
+                                                            PathBuf::from(target_dir.trim());
+                                                        print_warning(
+                                                            target_path.to_str().unwrap(),
+                                                            y,
+                                                        );
+                                                        state.current_dir =
+                                                            target_path.canonicalize()?;
+                                                        nums.reset();
+                                                        state.update_list()?;
+                                                        p_memo_v = Vec::new();
+                                                        c_memo_v = Vec::new();
+                                                        clear_and_show(&state.current_dir);
+                                                        state.list_up(nums.skip);
+                                                        print!("{}", cursor::Hide);
+                                                        state.move_cursor(&nums, STARTING_POINT);
+                                                        break 'command;
+                                                    }
                                                 }
                                             }
+                                        } else {
+                                            print_warning("zoxide not installed?", y);
                                         }
                                     }
 
@@ -1207,6 +1270,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
 
                                     print!("{}", screen::ToAlternateScreen);
                                     if std::env::set_current_dir(&state.current_dir).is_err() {
+                                        print!("{}", screen::ToAlternateScreen);
                                         print!("{}", cursor::Hide,);
                                         print_warning("Cannot execute command", y);
                                         break 'command;
@@ -1343,9 +1407,16 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
         }
         screen.flush()?;
     }
+
     //When exits, restore the cursor
     print!("{}", cursor::Restore);
+    let state = state_run.lock().unwrap();
+    //If layout was refreshed, go back to main screen to restore the previous state
     state.write_session(session_file_path)?;
+    let mut screen = screen_run.lock().unwrap();
+    write!(screen, "{}", screen::ToMainScreen)?;
+    screen.suspend_raw_mode()?;
+    screen.flush()?;
     Ok(())
 }
 
