@@ -16,7 +16,6 @@ use std::process::{Command, ExitStatus};
 use termion::{clear, color, cursor, style};
 
 pub const STARTING_POINT: u16 = 3;
-pub const TIME_WIDTH: u16 = 17;
 pub const DOWN_ARROW: char = '\u{21D3}';
 pub const RIGHT_ARROW: char = '\u{21D2}';
 pub const FX_CONFIG_DIR: &str = "felix";
@@ -96,10 +95,12 @@ pub enum FileType {
 
 #[derive(Clone)]
 pub struct Layout {
+    pub y: u16,
     pub terminal_row: u16,
     pub terminal_column: u16,
     pub name_max_len: usize,
     pub time_start_pos: u16,
+    pub use_full: Option<bool>,
     pub option_name_len: Option<usize>,
 }
 
@@ -112,39 +113,8 @@ impl Default for State {
             error!("Too small terminal size.");
             panic!("Panic due to terminal size (less than 21 columns).")
         };
-        let mut time_start: u16;
-        let mut name_max: usize;
-        match config.use_full_width {
-            Some(true) => {
-                time_start = column - 16;
-                name_max = (time_start - 3).into();
-            }
-            Some(false) | None => match config.item_name_length {
-                Some(option_max) => {
-                    time_start = option_max as u16 + 3;
-                    name_max = option_max;
-                }
-                None => {
-                    time_start = if column >= 49 {
-                        33
-                    } else {
-                        column - TIME_WIDTH
-                    };
-                    name_max = if column >= 49 {
-                        30
-                    } else {
-                        (time_start - 3).into()
-                    };
-                }
-            },
-        }
-
-        let required = time_start + TIME_WIDTH - 1;
-        if required > column {
-            let diff = required - column;
-            name_max -= diff as usize;
-            time_start -= diff;
-        }
+        let (time_start, name_max) =
+            make_layout(column, config.use_full_width, config.item_name_length);
 
         State {
             list: Vec::new(),
@@ -160,10 +130,12 @@ impl Default for State {
             commands: to_extension_map(&config.exec),
             sort_by: session.sort_by,
             layout: Layout {
+                y: STARTING_POINT,
                 terminal_row: row,
                 terminal_column: column,
                 name_max_len: name_max,
                 time_start_pos: time_start,
+                use_full: config.use_full_width,
                 option_name_len: config.item_name_length,
             },
             show_hidden: session.show_hidden,
@@ -176,6 +148,26 @@ impl State {
     pub fn new() -> Self {
         Default::default()
     }
+
+    pub fn refresh(&mut self, column: u16, row: u16, nums: &Num, cursor_pos: u16) {
+        let (time_start, name_max) =
+            make_layout(column, self.layout.use_full, self.layout.option_name_len);
+
+        *self = State {
+            layout: Layout {
+                terminal_row: row,
+                terminal_column: column,
+                name_max_len: name_max,
+                time_start_pos: time_start,
+                ..self.layout
+            },
+            ..self.to_owned()
+        };
+        clear_and_show(&self.current_dir);
+        self.list_up(nums.skip);
+        self.move_cursor(nums, cursor_pos);
+    }
+
     pub fn get_item(&self, index: usize) -> Result<&ItemInfo, MyError> {
         self.list.get(index).ok_or_else(|| {
             MyError::IoError(std::io::Error::new(
@@ -237,7 +229,7 @@ impl State {
                 });
             }
 
-            self.to_registered_mut(&item, to, rename);
+            self.push_to_registered(&item, to, rename);
 
             //remove original
             if std::fs::remove_file(from).is_err() {
@@ -313,7 +305,7 @@ impl State {
             }
         }
 
-        self.to_registered_mut(&item, trash_path, trash_name);
+        self.push_to_registered(&item, trash_path, trash_name);
 
         //remove original
         if std::fs::remove_dir_all(&item.file_path).is_err() {
@@ -325,7 +317,7 @@ impl State {
         Ok(())
     }
 
-    fn to_registered_mut(&mut self, item: &ItemInfo, file_path: PathBuf, file_name: String) {
+    fn push_to_registered(&mut self, item: &ItemInfo, file_path: PathBuf, file_name: String) {
         let mut buf = item.clone();
         buf.file_path = file_path;
         buf.file_name = file_name;
@@ -655,7 +647,7 @@ impl State {
         }
     }
 
-    pub fn move_cursor(&self, nums: &Num, y: u16) {
+    pub fn move_cursor(&mut self, nums: &Num, y: u16) {
         print!("{}", cursor::Goto(1, self.layout.terminal_row));
         print!("{}", clear::CurrentLine);
 
@@ -688,6 +680,7 @@ impl State {
             }
         }
         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
+        self.layout.y = y;
     }
 
     pub fn write_session(&self, session_path: PathBuf) -> Result<(), MyError> {
